@@ -75,7 +75,6 @@ class Routine(list):
     def writeStartCode(self, buff):
         """This is start of the *experiment* (before window is created)
         """
-        # few components will have this
         for thisCompon in self:
             # check just in case; try to ensure backwards compatibility _base
             if hasattr(thisCompon, 'writeStartCode'):
@@ -90,6 +89,16 @@ class Routine(list):
             if hasattr(thisCompon, 'writeStartCodeJS'):
                 thisCompon.writeStartCodeJS(buff)
 
+    def writeRunOnceInitCode(self, buff):
+        """ Run once init code goes at the beginning of the script (before
+        Window creation) and the code will be run only once no matter how many
+        similar components request it
+        """
+        for thisCompon in self:
+            # check just in case; try to ensure backwards compatibility _base
+            if hasattr(thisCompon, 'writeRunOnceInitCode'):
+                thisCompon.writeRunOnceInitCode(buff)
+
     def writeInitCode(self, buff):
         code = '\n# Initialize components for Routine "%s"\n'
         buff.writeIndentedLines(code % self.name)
@@ -107,16 +116,15 @@ class Routine(list):
             if hasattr(thisCompon, 'writeInitCodeJS'):
                 thisCompon.writeInitCodeJS(buff)
 
-    def writeResourcesCodeJS(self, buff):
-        buff.writeIndented("// <<maybe need to load images for {}?>>\n"
-                           .format(self.name))
-
     def writeMainCode(self, buff):
         """This defines the code for the frames of a single routine
         """
         # create the frame loop for this routine
         code = ('\n# ------Prepare to start Routine "%s"-------\n')
         buff.writeIndentedLines(code % (self.name))
+        code = 'continueRoutine = True\n'
+        buff.writeIndentedLines(code)
+
         # can we use non-slip timing?
         maxTime, useNonSlip = self.getMaxTime()
         if useNonSlip:
@@ -147,7 +155,6 @@ class Routine(list):
                 '_timeToFirstFrame = win.getFutureFlipTime(clock="now")\n'
                 '{clockName}.reset(-_timeToFirstFrame)  # t0 is time of first possible flip\n'
                 'frameN = -1\n'
-                'continueRoutine = True\n'
                 '\n# -------Run Routine "{name}"-------\n')
         buff.writeIndentedLines(code.format(name=self.name,
                                             clockName=self._clockName))
@@ -230,9 +237,13 @@ class Routine(list):
     def writeRoutineBeginCodeJS(self, buff, modular):
 
         # create the frame loop for this routine
-        code = ("\nfunction %(name)sRoutineBegin() {\n" % self.params)
+
+        code = ("\nfunction %(name)sRoutineBegin(trials) {\n" % self.params)
         buff.writeIndentedLines(code)
         buff.setIndentLevel(1, relative=True)
+        buff.writeIndentedLines("return function () {\n")
+        buff.setIndentLevel(1, relative=True)
+
         code = ("//------Prepare to start Routine '%(name)s'-------\n"
                 "t = 0;\n"
                 "%(name)sClock.reset(); // clock\n"
@@ -272,17 +283,22 @@ class Routine(list):
 
         buff.writeIndentedLines(code)
         buff.setIndentLevel(-1, relative=True)
+        buff.writeIndentedLines("};\n")
+        buff.setIndentLevel(-1, relative=True)
         buff.writeIndentedLines("}\n")
 
-
     def writeEachFrameCodeJS(self, buff, modular):
         # can we use non-slip timing?
         maxTime, useNonSlip = self.getMaxTime()
 
         # write code for each frame
-        code = ("\nfunction %(name)sRoutineEachFrame() {\n" % self.params)
+
+        code = ("\nfunction %(name)sRoutineEachFrame(trials) {\n" % self.params)
         buff.writeIndentedLines(code)
         buff.setIndentLevel(1, relative=True)
+        buff.writeIndentedLines("return function () {\n")
+        buff.setIndentLevel(1, relative=True)
+
         code = ("//------Loop for each frame of Routine '%(name)s'-------\n"
                 "let continueRoutine = true; // until we're told otherwise\n"
                 "// get current time\n"
@@ -303,9 +319,8 @@ class Routine(list):
 
         if self.exp.settings.params['Enable Escape'].val:
             code = ("// check for quit (typically the Esc key)\n"
-                    "if (psychoJS.experiment.experimentEnded "
-                    "|| psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {\n"
-                    "  return psychoJS.quit('The [Escape] key was pressed. Goodbye!', false);\n"
+                    "if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {\n"
+                    "  return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);\n"
                     "}\n\n")
             buff.writeIndentedLines(code)
 
@@ -329,7 +344,8 @@ class Routine(list):
             code = ("%(name)sComponents.forEach( function(thisComponent) {\n"
                     "  if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {\n"
                     "    continueRoutine = true;\n"
-                    "  }});\n")
+                    "  }\n"
+                    "});\n")
         buff.writeIndentedLines(code % self.params)
 
         buff.writeIndentedLines("\n// refresh the screen if continuing\n")
@@ -339,12 +355,12 @@ class Routine(list):
         else:
             buff.writeIndentedLines("if (continueRoutine) {")
         code = ("  return Scheduler.Event.FLIP_REPEAT;\n"
-                "}\n"
-                "else {\n"
+                "} else {\n"
                 "  return Scheduler.Event.NEXT;\n"
                 "}\n")
         buff.writeIndentedLines(code)
-
+        buff.setIndentLevel(-1, relative=True)
+        buff.writeIndentedLines("};\n")
         buff.setIndentLevel(-1, relative=True)
         buff.writeIndentedLines("}\n")
 
@@ -352,9 +368,11 @@ class Routine(list):
         # can we use non-slip timing?
         maxTime, useNonSlip = self.getMaxTime()
 
-        code = ("\nfunction %(name)sRoutineEnd() {\n" % self.params)
+        code = ("\nfunction %(name)sRoutineEnd(trials) {\n" % self.params)
         buff.writeIndentedLines(code)
         buff.setIndentLevel(1, relative=True)
+        buff.writeIndentedLines("return function () {\n")
+        buff.setIndentLevel(1, relative=True)
 
         if modular:
             code = ("//------Ending Routine '%(name)s'-------\n"
@@ -368,7 +386,8 @@ class Routine(list):
                     "%(name)sComponents.forEach( function(thisComponent) {\n"
                     "  if (typeof thisComponent.setAutoDraw === 'function') {\n"
                     "    thisComponent.setAutoDraw(false);\n"
-                    "  }});\n")
+                    "  }\n"
+                    "});\n")
         buff.writeIndentedLines(code  % self.params)
         # add the EndRoutine code for each component
         for compon in self:
@@ -384,6 +403,8 @@ class Routine(list):
 
         buff.writeIndented('return Scheduler.Event.NEXT;\n')
         buff.setIndentLevel(-1, relative=True)
+        buff.writeIndentedLines("};\n")
+        buff.setIndentLevel(-1, relative=True)
         buff.writeIndentedLines("}\n")
 
     def writeExperimentEndCode(self, buff):
