@@ -5,7 +5,7 @@
 '''
 
 # Part of the PsychoPy library
-# Copyright (C) 2018 Jonathan Peirce
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 from __future__ import absolute_import, division, print_function
@@ -94,8 +94,11 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
                  antialias=True,
                  bold=False,
                  italic=False,
-                 alignHoriz='center',
-                 alignVert='center',
+                 alignHoriz=None,
+                 alignVert=None,
+                 alignText='center',
+                 anchorHoriz='center',
+                 anchorVert='center',
                  fontFiles=(),
                  wrapWidth=None,
                  flipHoriz=False,
@@ -125,17 +128,18 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
             Apply settings to correctly display content from some languages
             that are written right-to-left. Currently there are three (case-
             insensitive) values for this parameter:
-            ``'LTR'`` is the default, for typical left-to-right, Latin-style
-            languages.
-            ``'RTL'`` will correctly display text in right-to-left languages
-             such as Hebrew. By applying the bidirectional algorithm, it
-             allows mixing portions of left-to-right content (such as numbers
-             or Latin script) within the string.
-            ``'Arabic'`` applies the bidirectional algorithm but additionally
-            will _reshape_ Arabic characters so they appear in the cursive,
-            linked form that depends on neighbouring characters, rather than
-            in their isolated form. May also be applied in other scripts,
-            such as Farsi or Urdu, that use Arabic-style alphabets.
+
+            - ``'LTR'`` is the default, for typical left-to-right, Latin-style
+                languages.
+            - ``'RTL'`` will correctly display text in right-to-left languages
+                such as Hebrew. By applying the bidirectional algorithm, it
+                allows mixing portions of left-to-right content (such as numbers
+                or Latin script) within the string.
+            - ``'Arabic'`` applies the bidirectional algorithm but additionally
+                will _reshape_ Arabic characters so they appear in the cursive,
+                linked form that depends on neighbouring characters, rather than
+                in their isolated form. May also be applied in other scripts,
+                such as Farsi or Urdu, that use Arabic-style alphabets.
 
         :Parameters:
 
@@ -164,8 +168,6 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         self._needVertexUpdate = True
         # use shaders if available by default, this is a good thing
         self.__dict__['useShaders'] = win._haveShaders
-        self.__dict__['alignHoriz'] = alignHoriz
-        self.__dict__['alignVert'] = alignVert
         self.__dict__['antialias'] = antialias
         self.__dict__['font'] = font
         self.__dict__['bold'] = bold
@@ -179,6 +181,24 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         self.__dict__['languageStyle'] = languageStyle
         self._pygletTextObj = None
         self.__dict__['pos'] = numpy.array(pos, float)
+        # deprecated attributes
+        if alignVert:
+            self.__dict__['alignVert'] = alignVert
+            logging.warning("TextStim.alignVert is deprecated. Use the "
+                            "anchorVert attribute instead")
+            # for compatibility, alignText was historically 'left'
+            anchorVert = alignHoriz
+        if alignHoriz:
+            self.__dict__['alignHoriz'] = alignHoriz
+            logging.warning("TextStim.alignHoriz is deprecated. Use alignText "
+                            "and anchorHoriz attributes instead")
+            # for compatibility, alignText was historically 'left'
+            alignText, anchorHoriz = alignHoriz, alignHoriz
+        # alignment and anchors
+        self.alignText = alignText
+        self.anchorHoriz = anchorHoriz
+        self.anchorVert = anchorVert
+
 
         # generate the texture and list holders
         self._listID = GL.glGenLists(1)
@@ -357,21 +377,16 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         """Set the text to be rendered using the current font
         """
         if self.win.winType in ["pyglet", "glfw"]:
-            self._pygletTextObj = pyglet.font.Text(
-                self._font, self.text,
-                halign=self.alignHoriz, valign=self.alignVert,
-                color=(1.0, 1.0, 1.0, self.opacity),
-                width=self._wrapWidthPix)  # width of the frame
-            # self._pygletTextObj = pyglet.text.Label(
-            #       self.text,self.font, int(self._heightPix),
-            #      anchor_x=self.alignHoriz,
-            #      anchor_y=self.alignVert,  # the point we rotate around
-            #      halign=self.alignHoriz,
-            #      color = (int(127.5 * self.rgb[0] + 127.5),
-            #               int(127.5 * self.rgb[1] + 127.5),
-            #               int(127.5 * self.rgb[2] + 127.5),
-            #               int(255 * self.opacity)),
-            # multiline=True, width=self._wrapWidthPix)  # width of the frame
+            self._pygletTextObj = pyglet.text.Label(
+                self.text, self.font, int(self._heightPix*0.75),
+                anchor_x=self.anchorHoriz,
+                anchor_y=self.anchorVert,  # the point we rotate around
+                align=self.alignText,
+                color = (int(127.5 * self.rgb[0] + 127.5),
+                      int(127.5 * self.rgb[1] + 127.5),
+                      int(127.5 * self.rgb[2] + 127.5),
+                      int(255 * self.opacity)),
+                multiline=True, width=self._wrapWidthPix)  # width of the frame
             self.width = self._pygletTextObj.width
             self._fontHeightPix = self._pygletTextObj.height
         else:
@@ -491,15 +506,17 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         desiredRGB = self._getDesiredRGB(self.rgb, self.colorSpace,
                                          self.contrast)
         if self.win.winType in ["pyglet", "glfw"]:
-            self._pygletTextObj = pyglet.font.Text(
-                self._font, self.text,
-                halign=self.alignHoriz, valign=self.alignVert,
-                color=(desiredRGB[0], desiredRGB[1], desiredRGB[2],
-                       self.opacity),
-                width=self._wrapWidthPix)  # width of the frame
-
+            self._pygletTextObj = pyglet.text.Label(
+                self.text, self.font, int(self._heightPix*0.75),
+                anchor_x=self.anchorHoriz,
+                anchor_y=self.anchorVert,  # the point we rotate around
+                align=self.alignText,
+                color = (int(127.5 * self.rgb[0] + 127.5),
+                      int(127.5 * self.rgb[1] + 127.5),
+                      int(127.5 * self.rgb[2] + 127.5),
+                      int(255 * self.opacity)),
+                multiline=True, width=self._wrapWidthPix)  # width of the frame
             self.width = self._pygletTextObj.width
-            self._fontHeightPix = self._pygletTextObj.height
         else:
             self._surf = self._font.render(value, self.antialias,
                                            [desiredRGB[0] * 255,
@@ -570,11 +587,11 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
             GL.glEnable(GL.GL_TEXTURE_2D)
         else:
             # bind the appropriate main texture
-            GL.glActiveTextureARB(GL.GL_TEXTURE0_ARB)
+            GL.glActiveTexture(GL.GL_TEXTURE0)
             GL.glEnable(GL.GL_TEXTURE_2D)
             GL.glBindTexture(GL.GL_TEXTURE_2D, self._texID)
             # unbind the mask texture regardless
-            GL.glActiveTextureARB(GL.GL_TEXTURE1_ARB)
+            GL.glActiveTexture(GL.GL_TEXTURE1)
             GL.glEnable(GL.GL_TEXTURE_2D)
             GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
 
@@ -657,22 +674,43 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         self.__dict__['italic'] = value
         self.font = self.font  # call attributeSetter
 
+    @attributeSetter
+    def alignHoriz(self, value):
+        """Deprecated in PsychoPy 3.3. Use `alignText` and `anchorHoriz`
+        instead
+        """
+        self.__dict__['alignHoriz'] = value
+        self._needSetText = True
+
     @attributeSetter
     def alignVert(self, value):
-        """The vertical alignment ('top', 'bottom' or 'center')
-        Note that this will not necessarily center the particular characters
-        you choose to draw. Instead, it likely centers the invisible
-        bounding box (which is often spanned by the top of a 'T' to the
-        bottom of a 'y') at the `pos`.
+        """Deprecated in PsychoPy 3.3. Use `anchorVert`
         """
         self.__dict__['alignVert'] = value
         self._needSetText = True
 
     @attributeSetter
-    def alignHoriz(self, value):
+    def alignText(self, value):
+        """Aligns the text content within the bounding box ('left', 'right' or
+        'center')
+        See also `anchorX` to set alignment of the box itself relative to pos
+        """
+        self.__dict__['alignText'] = value
+        self._needSetText = True
+
+    @attributeSetter
+    def anchorHoriz(self, value):
         """The horizontal alignment ('left', 'right' or 'center')
         """
-        self.__dict__['alignHoriz'] = value
+        self.__dict__['anchorHoriz'] = value
+        self._needSetText = True
+
+    @attributeSetter
+    def anchorVert(self, value):
+        """The vertical alignment ('top', 'bottom' or 'center') of the box
+        relative to the text `pos`.
+        """
+        self.__dict__['anchorVert'] = value
         self._needSetText = True
 
     @attributeSetter
@@ -722,8 +760,13 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         NOTE: currently always returns the size in pixels
         (this will change to return in stimulus units)
         """
-        return (self._pygletTextObj._layout.content_width,
-                self._pygletTextObj._layout.content_height)
+        if hasattr(self._pygletTextObj, 'content_width'):
+            w, h = (self._pygletTextObj.content_width,
+                    self._pygletTextObj.content_height)
+        else:
+            w, h = (self._pygletTextObj._layout.content_width,
+                    self._pygletTextObj._layout.content_height)
+        return w, h
 
     @property
     def posPix(self):
@@ -793,12 +836,6 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         if win.winType in ["pyglet", "glfw"]:
             if self._needSetText:
                 self.setText()
-            # and align based on x anchor
-            if self.alignHoriz == 'right':
-                GL.glTranslatef(-self.width, 0, 0)  # NB depth is set already
-            if self.alignHoriz in ['center', 'centre']:
-                # NB depth is set already
-                GL.glTranslatef(-self.width/2, 0, 0)
 
             # unbind the mask texture regardless
             GL.glActiveTexture(GL.GL_TEXTURE1)
